// perl.cpp : implementation file
//
// PM 08/07/09 Adding PERL_SYS_INIT3 and PERL_SYS_TERM
// PM 08/07/09 Adding a define for try catch

#ifdef WIN32
#pragma warning( disable : 4996 )
#endif

#include <string>
#include <map>
#include <vector>
#include <iostream>
#include <EXTERN.h>
#include <perl.h>
#include <XSUB.h>
#include "eperl.h"
#include "../nvs/nvperl.h"


using namespace std;

#define RESULT_BUFFER_LENGTH	1024
// Set the next define to display debug output on the console
//#define DEBUG_PERL
//#define DEBUG_ERROR_ONLY

#define USE_TRY_CATCH


EXTERN_C void xs_init (pTHX);
EXTERN_C void boot_DynaLoader (pTHX_ CV* cv);
//EXTERN_C void boot_Socket (pTHX_ CV* cv);

EXTERN_C XS(NV_SetErrorMode);
EXTERN_C XS(NV_SetError);
EXTERN_C XS(NV_SetErrorEx);
EXTERN_C XS(NV_Command);
EXTERN_C XS(NV_GetSessionId);
EXTERN_C XS(NV_GetService);
EXTERN_C XS(NV_GetClass);
EXTERN_C XS(NV_GetCommand);
EXTERN_C XS(NV_GetInContainer);
EXTERN_C XS(NV_GetOutContainer);
EXTERN_C XS(NV_GetLanguage);
EXTERN_C XS(NV_GetError);
EXTERN_C XS(NV_GetNumParameters);
EXTERN_C XS(NV_GetSource);
EXTERN_C XS(NV_ParameterExist);
EXTERN_C XS(NV_GetParameter);
EXTERN_C XS(NV_GetParameterName);
EXTERN_C XS(NV_SetEncoding);
EXTERN_C XS(NV_EvalString);


// Classes declared in this file
class NvPerlInterpretor;
class NvPerlInterpretorLists;
class NvMutex;

NvPerlInterpretor* GetInterpretorFromCallBack();

map<void*, void*> InterpretorLists;
char PerlLibPath[512];
char NirvaDir[512];

static PerlInterpreter *global_perl_session = NULL;

// NvPerlInterpretor class
class NvPerlInterpretor
{
public:		
	NvPerlCommand *m_pCommand;					// Pointer to a Nirva command object.
	PerlInterpreter *perl_session;
	char *ResultBuffer;
	int ResultBufferSize;
	string ScriptFile;
	string ScriptString;
	int ErrorModeScript;
	int GotError;
	string Id;		// Used for logs
	int LogLevel;
	bool bDebug;
	string EvalError;

	NvPerlInterpretor();
	virtual ~NvPerlInterpretor();


public:		

	int Check(int a, int b);

	// RunScript
	int RunScript(int Debug = 0);
	int RunScriptMemory(int Debug = 0);
	
	// Send a command to Nirva
	int Command(const char *CommandString, char **ResBuffer);

	// Set the error information to Nirva
	void SetError(const char *ErrorInfo);
	void SetErrorEx(const char *ErrorService, const char *ErrorClass, int ErrorCode, const char *ErrorInfo);

	int SetResultBufSize(int DataSize);
	void GetSessionId(char **ResBuffer);
	void GetService(char **ResBuffer);
	void GetClass(char **ResBuffer);
	void GetCommand(char **ResBuffer);
	void GetInContainer(char **ResBuffer);
	void GetOutContainer(char **ResBuffer);
	void GetLanguage(char **ResBuffer);
	void GetError(const char *InfoType, char **ResBuffer);
	int GetNumParameters();
	int GetSource();
	int ParameterExist(const char *ParameterName);
	void GetParameter(const char *ParameterName, char **ResBuffer);
	void GetParameterName(int index, char **ResBuffer);
	void SetEncoding(const char *Encoding);
};



/////////////////////////////////////////////////////////////////////////////
// NvPerlInterpretorList class
class NvPerlInterpretorList
{
public:		
	vector<void*> m_vInterpretors;		// List of interpretors

	NvPerlInterpretorList();
	virtual ~NvPerlInterpretorList();
};

// NvMutex class
// A mutex can just be locked or unlocked
// It allows thread protection of some part of code
class NvMutex
{
private:

	friend class NvEvent;	// NvEvent should access private members
// mutext declaration
#ifdef WIN32
	HANDLE m_mMutex;
#else
	pthread_mutex_t m_mMutex;
#endif

public:
	// standard constructor
	NvMutex();

	// copy constructor
	// this in fact don't do anything but avoid copying of the mutex object handle
	NvMutex(const NvMutex& MutexSrc);

	// destructor
	virtual ~NvMutex();

	// Locks the mutex
	void Lock();

	// Unlocks the mutex
	void Unlock();
	
	// We overload the assignement operator in order to not allow copy of the mutex handle
	const NvMutex& operator = (const NvMutex& MutexSrc);
};

NvMutex ProtectInterpretors;

#ifdef DEBUG_PERL
NvMutex ProtectDebug;
#ifdef DEBUG_ERROR_ONLY
#define DEBUGOUTPUT(O)
#else
#define DEBUGOUTPUT(O)							{ProtectDebug.Lock();\
											cout << "\nPerl debug : " << O << "  -  " << __LINE__ << "\n" << flush;\
											ProtectDebug.Unlock();}
#endif
#define DEBUGERROR(O)							{ProtectDebug.Lock();\
											cout << "\nPerl debug : " << O << "  -  " << __LINE__ << "\n" << flush;\
											ProtectDebug.Unlock();}
#else
#define DEBUGOUTPUT(O)
#define DEBUGERROR(O)
#endif


// NvMutex class

// constructor
NvMutex::NvMutex()
{
#ifdef WIN32
	m_mMutex = CreateMutex(NULL, 0, NULL);
#else
	pthread_mutex_init(&m_mMutex, NULL);
#endif
}


// constructor by copy
// we don't accept to copy a mutex, we create another one
NvMutex::NvMutex(const NvMutex& MutexSrc)
{
#ifdef WIN32
	m_mMutex = CreateMutex(NULL, 0, NULL);
#else
	pthread_mutex_init(&m_mMutex, NULL);
#endif
}


// destructor
NvMutex::~NvMutex()
{
#ifdef WIN32
	ReleaseMutex(m_mMutex);
	CloseHandle(m_mMutex);
#else
	pthread_mutex_unlock(&m_mMutex);
	pthread_mutex_destroy(&m_mMutex);
#endif
}


// lock
void NvMutex::Lock()
{
#ifdef WIN32
	WaitForSingleObject(m_mMutex, INFINITE);
#else
	pthread_mutex_lock(&m_mMutex);
#endif
}


// unlock
void NvMutex::Unlock()
{
#ifdef WIN32
	ReleaseMutex(m_mMutex);
#else
	pthread_mutex_unlock(&m_mMutex);
#endif
}

// Assignement operator
const NvMutex& NvMutex::operator = (const NvMutex& MutexSrc)
{
	return *this;	// we do nothing
}

// convert a string to upper case
void MakeUpper(string& sString)
{
	for(int i=0; i<(int)(sString.length()); i++)
		sString.at(i) = (char)(toupper(sString[i]));
}
// convert a string to lower case
void MakeLower(string& sString)
{
	for(int i=0; i<(int)sString.length(); i++)
		sString.at(i) = (char)(tolower(sString[i]));
}


// make some cleaning on strings
void CleanString(string& StringToClean)
{
	if(!StringToClean.size())
		return;
	
	string::size_type index;
	
	// replace tab with space
	while((index = StringToClean.find('\t')) != string::npos)
		StringToClean.replace(index, 1, 1, ' ');
	// remove line feeds
	while((index = StringToClean.find('\n')) != string::npos)
		StringToClean.erase(index, 1);
	// remove carriage returns
	while((index = StringToClean.find('\r')) != string::npos)
		StringToClean.erase(index, 1);
	// concatenate 2 consecutive spaces
	while((index = StringToClean.find("  ")) != string::npos)
		StringToClean.replace(index, 2, " ");
	// eliminate first space
	if(StringToClean.size() && StringToClean[0] == ' ')
		StringToClean.erase(0, 1);
	// eliminate last space
	if(StringToClean.size() && StringToClean[StringToClean.size()-1] == ' ')
		StringToClean.erase(StringToClean.size()-1, 1);
	
}


/////////////////////////////////////////////////////////////////////////////
// NvPerlInterpretorList

NvPerlInterpretorList::NvPerlInterpretorList()
{
}


NvPerlInterpretorList::~NvPerlInterpretorList()
{
	int Size = (int)(m_vInterpretors.size());
	int i;
	for(i=0;i<Size;i++)
	{
		NvPerlInterpretor *PerlInterpretor = (NvPerlInterpretor *)(m_vInterpretors[i]);
		try
		{
			if(PerlInterpretor != NULL)
				delete PerlInterpretor;
		}
		catch(...)
		{
		}
	}
	m_vInterpretors.clear();
}
	


/////////////////////////////////////////////////////////////////////////////
// NvPerlInterpretor

NvPerlInterpretor::NvPerlInterpretor()
{
	ResultBuffer = NULL;
	ResultBufferSize = 0;
	ErrorModeScript = 0;
	GotError = 0;
	perl_session = NULL;
	ScriptFile = "";
	ScriptString = "";
	char Buf[50];
	sprintf(Buf, "%08X", (int)((long)this)); 
	Id = Buf;
	LogLevel = 0;
	bDebug = false;
	EvalError = "";
}


// The ProtectInterpretors mutext must be locked  when calling the destructor
// Except if we are sure that perl_session is NULL
NvPerlInterpretor::~NvPerlInterpretor()
{
	if(perl_session != NULL)
	{
		PERL_SET_CONTEXT(perl_session);
#ifdef USE_TRY_CATCH
		try
		{
#endif
			perl_destruct(perl_session);
#ifdef USE_TRY_CATCH
		}
		catch(...)
		{
			DEBUGERROR("*** NvPerlInterpretor::Destructor");
		}
#endif
		DEBUGOUTPUT("NvPerlInterpretor::Destructor");
#ifdef USE_TRY_CATCH
		try
		{
#endif
			perl_free(perl_session);
#ifdef USE_TRY_CATCH
		}
		catch(...)
		{
			DEBUGERROR("*** NvPerlInterpretor::Destructor");
		}
#endif
		perl_session = NULL;
	}

	if(ResultBufferSize)
		free(ResultBuffer);
}

int NvPerlInterpretor::Check(int a, int b)
{
	return a + b;
}



// Run the perl script
int NvPerlInterpretor::RunScript(int Debug)
{
	if(Debug)
		bDebug = true;
	else
		bDebug = false;

	ProtectInterpretors.Lock();
	// We have a global perl session from which we create clones
	if(global_perl_session == NULL)
	{
		if(LogLevel)
			m_pCommand->SysLog("First call to perl - creating global interpretor", "", NV_LOG_INFO, 2, Id.c_str());
		char *Globalembedding[] = { (char *)"", (char *)"-e", (char *)"0" };
		global_perl_session = perl_alloc();
		perl_construct(global_perl_session);
		perl_parse(global_perl_session, NULL, 3, Globalembedding, (char **)NULL);
//		perl_run(global_perl_session);
		PL_perl_destruct_level = 2;
		if(LogLevel)
			m_pCommand->SysLog("Global interpretor creation successful", "", NV_LOG_INFO, 2, Id.c_str());
	}
	if(SetResultBufSize(0) == 0)
	{
		ProtectInterpretors.Unlock();
		if(LogLevel)
			m_pCommand->SysLog("*** Error in SetResultBufSize", "", NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}

	if(LogLevel)
		m_pCommand->SysLog("cloning interpretor", "", NV_LOG_INFO, 2, Id.c_str());
	UV clone_flags = 0;
#if defined(WIN32) && defined(CLONEf_CLONE_HOST)
	clone_flags |= CLONEf_CLONE_HOST;
#endif
	PERL_SET_CONTEXT(global_perl_session);
#ifdef USE_TRY_CATCH
	try
	{
#endif
		perl_session = perl_clone(global_perl_session, clone_flags);
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		perl_session = NULL;
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (cloning perl interpretor)|");
	}
#endif
	if(perl_session == NULL)
	{
		ProtectInterpretors.Unlock();
		if(LogLevel)
			m_pCommand->SysLog("*** Error when cloning perl interpretor", "", NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}

	PERL_SET_CONTEXT(perl_session);

	// PM Modif
	// From Nirva only the NvInterpretorRunScriptMemory is called from Nirva 3.0.001
	// So the ScriptFile is never given. This allows to block the STDOUT if not in debug mode
	char *argv[10];
	argv[0] = (char *)"";
	argv[1] = PerlLibPath;
	int argc = 2;
	if(!ScriptFile.length())
	{
		// Block STDIN if the script file is not given (we cannot use the -e option if a script file is given)
		argv[argc++] = (char *)("-e");
		argv[argc++] = (char *)("close(STDIN);");
	}
	if(Debug)
	{
		// Set warning mode in debug
		argv[argc++] = (char *)"-w";;
	}
	else
	{
		if(!ScriptFile.length())
		{
			// Block STDOUT and STDERR in normal mode (not debug)
			// if the script file is not given (we cannot use the -e option if a script file is given)
			argv[argc++] = (char *)("-e");
			argv[argc++] = (char *)("close(STDOUT);close(STDERR);close(STDIN);");
		}
	}
	if(ScriptFile.length())
		argv[argc++] = (char *)(ScriptFile.c_str());
	else
	{
		argv[argc++] = (char *)("-e");
		argv[argc++] = (char *)("if(!NV::EvalString()){exit(-1);}");
	}
	argv[argc] = 0;


	bool ParseResult = true;
	if(LogLevel)
		m_pCommand->SysLog("parsing script", "", NV_LOG_INFO, 2, Id.c_str());
#ifdef USE_TRY_CATCH
	try
	{
#endif
		if(perl_parse(perl_session, xs_init, argc, argv, (char **)NULL) != 0)
			ParseResult = false;
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_parse)|");
		ParseResult = false;
	}
#endif

	if(!ParseResult)
	{
		PERL_SET_CONTEXT(perl_session);
#ifdef USE_TRY_CATCH
		try
		{
#endif
			perl_destruct(perl_session);
			perl_free(perl_session);
#ifdef USE_TRY_CATCH
		}
		catch(...)
		{
			m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_destruct)|");
		}
#endif
		ParseResult = false;
		perl_session = NULL;
		if(ResultBufferSize)
			free(ResultBuffer);
		ResultBufferSize = 0;
		ProtectInterpretors.Unlock();
		if(LogLevel)
			m_pCommand->SysLog("*** Error when parsing perl script", "", NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}
	
	ProtectInterpretors.Unlock();
	

	if(LogLevel)
		m_pCommand->SysLog("running script", "", NV_LOG_INFO, 2, Id.c_str());
	int RetCode = 0;
	bool RetVal = true;
#ifdef USE_TRY_CATCH
	try
	{
#endif
		PERL_SET_CONTEXT(perl_session);
		RetCode = perl_run(perl_session);
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		RetVal = false;
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_run)|");
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		if(LogLevel)
			m_pCommand->SysLog("*** Exception occured when running script", "", NV_LOG_INFO, 2, Id.c_str());
	}
#endif
#ifdef USE_TRY_CATCH
	try
	{
#endif
		PERL_SET_CONTEXT(perl_session);
		perl_destruct(perl_session);
		perl_free(perl_session);
		perl_session = NULL;
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_destruct)|");
	}
#endif

	if(ResultBufferSize)
		free(ResultBuffer);
	ResultBufferSize = 0;

	if(!RetVal)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		return 0;
	}


	if(GotError)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		if(LogLevel)
			m_pCommand->SysLog("Script has generated a normal Nirva error", EvalError.c_str(), NV_LOG_INFO, 1, Id.c_str());
		return 0;
	}
	if(RetCode != 0)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		if(LogLevel)
			m_pCommand->SysLog("*** Error when running perl script", EvalError.c_str(), NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}

	return 1;
}




// Run the perl script
int NvPerlInterpretor::RunScriptMemory(int Debug)
{
	if(Debug)
		bDebug = true;
	else
		bDebug = false;

	ProtectInterpretors.Lock();
	// We have a global perl session from which we create clones
	if(global_perl_session == NULL)
	{
		if(LogLevel)
			m_pCommand->SysLog("First call to perl - creating global interpretor", "", NV_LOG_INFO, 2, Id.c_str());
		char *Globalembedding[] = { (char *)"", (char *)"-e", (char *)"0" };
		global_perl_session = perl_alloc();
		perl_construct(global_perl_session);
		perl_parse(global_perl_session, NULL, 3, Globalembedding, (char **)NULL);
//		perl_run(global_perl_session);
		PL_perl_destruct_level = 2;
		if(LogLevel)
			m_pCommand->SysLog("Global interpretor creation successful", "", NV_LOG_INFO, 2, Id.c_str());
	}
	if(SetResultBufSize(0) == 0)
	{
		ProtectInterpretors.Unlock();
		if(LogLevel)
			m_pCommand->SysLog("*** Error in SetResultBufSize", "", NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}

	if(LogLevel)
		m_pCommand->SysLog("cloning interpretor", "", NV_LOG_INFO, 2, Id.c_str());
	UV clone_flags = 0;
#if defined(WIN32) && defined(CLONEf_CLONE_HOST)
	clone_flags |= CLONEf_CLONE_HOST;
#endif
	PERL_SET_CONTEXT(global_perl_session);
#ifdef USE_TRY_CATCH
	try
	{
#endif
		perl_session = perl_clone(global_perl_session, clone_flags);
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		perl_session = NULL;
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_clone)|");
	}
#endif
	if(perl_session == NULL)
	{
		ProtectInterpretors.Unlock();
		if(LogLevel)
			m_pCommand->SysLog("*** Error when cloning perl interpretor", "", NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}

	PERL_SET_CONTEXT(perl_session);

	char *argv[5];
	argv[0] = (char *)"";
	argv[1] = PerlLibPath;
	int argc = 3;
	if(Debug)
	{
		argv[2] = (char *)"-w";;
		argv[3] = (char *)("-e");
		argv[4] = 0;
		argc = 4;
	}
	else
	{
		argv[2] = (char *)("-e");
		argv[3] = 0;
	}

	bool ParseResult = true;
	if(LogLevel)
		m_pCommand->SysLog("parsing script", "", NV_LOG_INFO, 2, Id.c_str());
#ifdef USE_TRY_CATCH
	try
	{
#endif
		if(perl_parse(perl_session, xs_init, argc, argv, (char **)NULL) != 0)
			ParseResult = false;
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_parse)|");
		ParseResult = false;
	}
#endif

	if(!ParseResult)
	{
		PERL_SET_CONTEXT(perl_session);
#ifdef USE_TRY_CATCH
		try
		{
#endif
			perl_destruct(perl_session);
			perl_free(perl_session);
#ifdef USE_TRY_CATCH
		}
		catch(...)
		{
			m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_destruct)|");
		}
#endif
		ParseResult = false;
		perl_session = NULL;
		if(ResultBufferSize)
			free(ResultBuffer);
		ResultBufferSize = 0;
		ProtectInterpretors.Unlock();
		if(LogLevel)
			m_pCommand->SysLog("*** Error when parsing perl memory script", "", NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}
	
	ProtectInterpretors.Unlock();
	

	if(LogLevel)
		m_pCommand->SysLog("running memory script", "", NV_LOG_INFO, 2, Id.c_str());
	int RetCode = 0;
	bool RetVal = true;
#ifdef USE_TRY_CATCH
	try
	{
#endif
		PERL_SET_CONTEXT(perl_session);
		RetCode = perl_run(perl_session);
		ENTER;
		SAVETMPS;
		SV *value = (eval_pv(ScriptString.c_str(), TRUE));
		if(SvTRUE(ERRSV))
			RetCode = 1;	// Error
		FREETMPS;
		LEAVE;
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		RetVal = false;
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (running memory script)|");
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		if(LogLevel)
			m_pCommand->SysLog("*** Exception occured when running memory script", "", NV_LOG_INFO, 2, Id.c_str());
	}
#endif
#ifdef USE_TRY_CATCH
	try
	{
#endif
		PERL_SET_CONTEXT(perl_session);
		perl_destruct(perl_session);
		perl_free(perl_session);
		perl_session = NULL;
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (perl_destruct)|");
	}
#endif

	if(ResultBufferSize)
		free(ResultBuffer);
	ResultBufferSize = 0;

	if(!RetVal)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		return 0;
	}
	if(RetCode != 0)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		if(LogLevel)
			m_pCommand->SysLog("*** Error when running perl memory script", "", NV_LOG_ERROR, 1, Id.c_str());
		return 0;
	}

	if(GotError)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		if(LogLevel)
			m_pCommand->SysLog(" Memory script has generated a normal Nirva error", "", NV_LOG_INFO, 1, Id.c_str());
		return 0;
	}

	return 1;
}




// Increases the output buffer if necessary
// returns 0 if failure
// returns 1 if no change or first allocation
// returns 2 if realloc
int NvPerlInterpretor::SetResultBufSize(int DataSize)
{
	if(DataSize < ResultBufferSize)
		return 1;

	if(ResultBufferSize == 0)
	{
		ResultBuffer = (char *)malloc(RESULT_BUFFER_LENGTH);
		if(ResultBuffer == NULL)
			return 0;
		ResultBufferSize = RESULT_BUFFER_LENGTH;
	}
	if(DataSize < ResultBufferSize)
		return 1;

	char *NewResultBuffer = (char *)malloc(DataSize+1);
	if(NewResultBuffer == NULL)
		return 0;

	free(ResultBuffer);
	ResultBuffer = NewResultBuffer;
	ResultBufferSize = DataSize+1;

	return 2;
}


// Execute a NIRVA command
int NvPerlInterpretor::Command(const char *CommandString, char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	bool bControlErrorOnScript = false;
	if(ErrorModeScript)
		bControlErrorOnScript = true;
	if(!m_pCommand->Command(CommandString, ResultBuffer, ResultBufferSize, &DataSize, bControlErrorOnScript))
	{
		PERL_SET_CONTEXT(perl_session);
		return 0;
	}
	int ResultSetResultBufSize = SetResultBufSize(DataSize);
	if(ResultSetResultBufSize == 0)
	{
		PERL_SET_CONTEXT(perl_session);
		return 0;
	}
	if(ResultSetResultBufSize == 2)
	{
		*ResBuffer = ResultBuffer;
		// We got a reallocation so we must get the last output buffer
		m_pCommand->Command("NV_CMD=|COMMAND:GET_LAST_OUTPUT_BUFFER|", ResultBuffer, ResultBufferSize, &DataSize, bControlErrorOnScript);
//		if(!m_pCommand->Command("", ResultBuffer, ResultBufferSize, &DataSize, bControlErrorOnScript))
//		{
//			PERL_SET_CONTEXT(perl_session);
//			return 0;
//		}
	}
	PERL_SET_CONTEXT(perl_session);
	return 1;
}


// SetError
void NvPerlInterpretor::SetError(const char *ErrorInfo)
{
	m_pCommand->SetError(ErrorInfo);
}


// SetErrorEx
void NvPerlInterpretor::SetErrorEx(const char *ErrorService, const char *ErrorClass, int ErrorCode, const char *ErrorInfo)
{
	m_pCommand->SetErrorEx(ErrorService, ErrorClass, ErrorCode, ErrorInfo);
}


void NvPerlInterpretor::GetSessionId(char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetSessionId(ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetSessionId(ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetService(char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetService(ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetService(ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetClass(char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetClass(ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetClass(ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetCommand(char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetCommand(ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetCommand(ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetInContainer(char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetInContainer(ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetInContainer(ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetOutContainer(char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetOutContainer(ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetOutContainer(ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetLanguage(char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetLanguage(ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetLanguage(ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetError(const char *InfoType, char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetError(InfoType, ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetError(InfoType, ResultBuffer, ResultBufferSize, &DataSize);
	}
}

int NvPerlInterpretor::GetNumParameters()
{
	return m_pCommand->GetNumParameters();
}

int NvPerlInterpretor::GetSource()
{
	return m_pCommand->GetSource();
}

int NvPerlInterpretor::ParameterExist(const char *ParameterName)
{
	if(m_pCommand->ParameterExist(ParameterName))
		return 1;

	return 0;
}

void NvPerlInterpretor::GetParameter(const char *ParameterName, char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetParameter(ParameterName, ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetParameter(ParameterName, ResultBuffer, ResultBufferSize, &DataSize);
	}
}


void NvPerlInterpretor::GetParameterName(int index, char **ResBuffer)
{
	*ResBuffer = ResultBuffer;
	strcpy(ResultBuffer, "");
	int DataSize;
	m_pCommand->GetParameterName(index, ResultBuffer, ResultBufferSize, &DataSize);
	if(SetResultBufSize(DataSize) == 2)
	{
		*ResBuffer = ResultBuffer;
		m_pCommand->GetParameterName(index, ResultBuffer, ResultBufferSize, &DataSize);
	}
}

void NvPerlInterpretor::SetEncoding(const char *Encoding)
{
	m_pCommand->SetEncoding(Encoding);
}


#ifdef WIN32
// Main dll entry point for windows
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
		
	return 1;   // ok
}
#endif


int InitPerlLib(const char* NirvaPath)
{
	strcpy(NirvaDir, NirvaPath);
	strcpy(PerlLibPath, "-I");
	if(NirvaPath == NULL)
		strcat(PerlLibPath, ".");
	else
		strcat(PerlLibPath, NirvaPath);

#ifdef WIN32
	if(PerlLibPath[strlen(PerlLibPath)-1] != '\\')
		strcat(PerlLibPath, "\\");
#else
	if(PerlLibPath[strlen(PerlLibPath)-1] != '/')
		strcat(PerlLibPath, "/");
#endif
	strcat(PerlLibPath, "Perl");

	int argc = 0;
	char **argv = (char **)NULL;
	char **env = (char **)NULL;
	PERL_SYS_INIT3(&argc, &argv, &env);
	return 1;
}

void ExitPerlLib()
{
	// Removes all the interpretors if not yet made
	ProtectInterpretors.Lock();
	map<void *,void *>::iterator pos;
	NvPerlInterpretorList *PerlClassList;
	for(pos = InterpretorLists.begin(); pos != InterpretorLists.end(); ++pos)
	{
		PerlClassList = (NvPerlInterpretorList *)(pos->second);
		if(PerlClassList == NULL)
			continue;
		try
		{
			if(PerlClassList != NULL)
				delete PerlClassList;
		}
		catch(...)
		{
		}
	}
	InterpretorLists.clear();

	// Removes also the global interpretor
	if(global_perl_session != NULL)
	{
		PERL_SET_CONTEXT(global_perl_session);
		perl_destruct(global_perl_session);
		perl_free(global_perl_session);
		global_perl_session = NULL;
	}
	ProtectInterpretors.Unlock();

	PERL_SYS_TERM();
}


NVPERLINTERPRETOR NvGetInterpretor()
{
	// We create a new interpretor 
	NvPerlInterpretor *PerlInterpretor = new NvPerlInterpretor;
	if(PerlInterpretor == NULL)
		return (NVPERLINTERPRETOR)NULL;
	DEBUGOUTPUT("NvGetInterpretor");
	
	NvPerlInterpretorList *PerlInterpretorList = NULL;
#ifdef WIN32
	void *ThreadHandle = (void *)GetCurrentThreadId();			// Thread handle for window
#else
	void *ThreadHandle = (void *) pthread_self();				// Thread handle for unix
#endif
	ProtectInterpretors.Lock();
	if(InterpretorLists.find(ThreadHandle) != InterpretorLists.end())
		PerlInterpretorList = (NvPerlInterpretorList *)(InterpretorLists[ThreadHandle]);
	if(PerlInterpretorList == NULL)
		PerlInterpretorList = new NvPerlInterpretorList;
	if(PerlInterpretorList == NULL)
	{
		delete PerlInterpretor;
		if(InterpretorLists.find(ThreadHandle) != InterpretorLists.end())
			InterpretorLists.erase(ThreadHandle);
		return (NVPERLINTERPRETOR)NULL;
	}
	PerlInterpretorList->m_vInterpretors.push_back((void *)PerlInterpretor);
	InterpretorLists[ThreadHandle] = (void *)PerlInterpretorList;
	ProtectInterpretors.Unlock();

	return (NVPERLINTERPRETOR)PerlInterpretor;
}


void NvFreeInterpretor(NVPERLINTERPRETOR Interpretor)
{
	// Note Interpretor is not used here since we keep a list of interpretors
	// for each thread and we must release the last of this list

#ifdef WIN32
	void *ThreadHandle = (void *)GetCurrentThreadId();			// Thread handle for window
#else
	void *ThreadHandle = (void *) pthread_self();				// Thread handle for unix
#endif

	ProtectInterpretors.Lock();
	NvPerlInterpretorList *PerlInterpretorList = NULL;
	if(InterpretorLists.find(ThreadHandle) != InterpretorLists.end())
		PerlInterpretorList = (NvPerlInterpretorList *)(InterpretorLists[ThreadHandle]);
	int Size = 0;
	if(PerlInterpretorList != NULL)
		Size = (int)(PerlInterpretorList->m_vInterpretors.size());
	if(Size)
	{
		NvPerlInterpretor *PerlInterpretor = (NvPerlInterpretor *)(PerlInterpretorList->m_vInterpretors[Size-1]);
		if(((void*)PerlInterpretor) == Interpretor)
		{
			PerlInterpretorList->m_vInterpretors.pop_back();
		}
		else
		{
			// This should not occure but let's process it in case of pb
			int i;
			for(i=0;i<Size;i++)
			{
				PerlInterpretor = (NvPerlInterpretor *)(PerlInterpretorList->m_vInterpretors[i]);
				if(((void*)PerlInterpretor) == Interpretor)
					break;
			}
			if(i==Size)
				PerlInterpretor = NULL;	// not found
			else
				PerlInterpretorList->m_vInterpretors.erase(PerlInterpretorList->m_vInterpretors.begin() + i);
		}
		try
		{
			if(PerlInterpretor != NULL)
				delete PerlInterpretor;
			DEBUGOUTPUT("NvFreeInterpretor");
		}
		catch(...)
		{
			DEBUGERROR("*** NvFreeInterpretor");
		}
		if(PerlInterpretorList->m_vInterpretors.size() == 0)
		{
			delete PerlInterpretorList;
			InterpretorLists.erase(ThreadHandle);
		}
	}
	ProtectInterpretors.Unlock();
}


int NvInterpretorRunScript(NVPERLINTERPRETOR Interpretor, const char* ScriptFileName, void *pCommand, int Debug)
{
	NvPerlCommand* pCommandHere = (NvPerlCommand*)pCommand;
	if(ScriptFileName == NULL)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		pCommandHere->SysLog("Run script", "", NV_LOG_INFO, 1);
		pCommandHere->SysLog("*** Error - Script file name is NULL", "", NV_LOG_ERROR, 1);
		return 0;
	}
	
	DEBUGOUTPUT("NvInterpretorRunScript");
	NvPerlInterpretor *PerlInterpretor = GetInterpretorFromCallBack();
	if(PerlInterpretor == NULL)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		pCommandHere->SysLog("*** Error - Cannot get interpretor", ScriptFileName, NV_LOG_ERROR, 1);
		return 0;
	}
	PerlInterpretor->LogLevel = pCommandHere->GetLogLevel();
	if(PerlInterpretor->LogLevel)
		pCommandHere->SysLog("Run script", ScriptFileName, NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
	PerlInterpretor->m_pCommand = pCommandHere;
	PerlInterpretor->ScriptFile = ScriptFileName;
	int RetVal = 1;
	DEBUGOUTPUT("NvInterpretorRunScript");
	try
	{
		RetVal = PerlInterpretor->RunScript(Debug);
	}
	catch(...)
	{
		pCommandHere->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (run script)|");
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		if(PerlInterpretor->LogLevel)
			pCommandHere->SysLog("*** Error - Exception when running script", ScriptFileName, NV_LOG_ERROR, 1, PerlInterpretor->Id.c_str());
		RetVal = 0;
	}
	
	if(PerlInterpretor->LogLevel)
	{
		if(RetVal)
			pCommandHere->SysLog("Script terminated successfuly", ScriptFileName, NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
		else
			pCommandHere->SysLog("*** Script terminated with error", ScriptFileName, NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
	}

	DEBUGOUTPUT("NvInterpretorRunScript");
	return RetVal;
}



int NvInterpretorRunScriptMemory(NVPERLINTERPRETOR Interpretor, const char* ScriptData, void *pCommand, int Debug)
{
	NvPerlCommand* pCommandHere = (NvPerlCommand*)pCommand;
	if(ScriptData == NULL)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		pCommandHere->SysLog("Run script", "", NV_LOG_INFO, 1);
		pCommandHere->SysLog("*** Error - Script data is NULL", "", NV_LOG_ERROR, 1);
		return 0;
	}
	DEBUGOUTPUT("NvInterpretorRunScriptMemory");
	NvPerlInterpretor *PerlInterpretor = GetInterpretorFromCallBack();
	if(PerlInterpretor == NULL)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		pCommandHere->SysLog("*** Error - Cannot get interpretor", "", NV_LOG_ERROR, 1);
		return 0;
	}
	PerlInterpretor->LogLevel = pCommandHere->GetLogLevel();
	if(PerlInterpretor->LogLevel)
		pCommandHere->SysLog("Run memory script", "", NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
	PerlInterpretor->m_pCommand = pCommandHere;
	PerlInterpretor->ScriptString = ScriptData;
	PerlInterpretor->ScriptFile = "";
//	PerlInterpretor->ScriptFile = NirvaDir;
//	PerlInterpretor->ScriptFile += "Procs/System/runperl.pl";
	int RetVal = 1;
	DEBUGOUTPUT("NvInterpretorRunScriptMemory");
//_CrtMemState s1;
//_CrtMemCheckpoint( &s1 );
	try
	{
		RetVal = PerlInterpretor->RunScript(Debug);
	}
	catch(...)
	{
		pCommandHere->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (run script memory)|");
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		if(PerlInterpretor->LogLevel)
			pCommandHere->SysLog("*** Error - Exception when running script", "", NV_LOG_ERROR, 1, PerlInterpretor->Id.c_str());
		RetVal = 0;
	}
//_CrtMemDumpAllObjectsSince(  &s1 );
	
	if(PerlInterpretor->LogLevel)
	{
		if(RetVal)
			pCommandHere->SysLog("Script terminated successfuly", "", NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
		else
			pCommandHere->SysLog("*** Script terminated with error", "", NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
	}

	DEBUGOUTPUT("NvInterpretorRunScriptMemory");
	return RetVal;
}

/*
int NvInterpretorRunScriptMemory(NVPERLINTERPRETOR Interpretor, const char* ScriptData, void *pCommand)
{
	NvPerlCommand* pCommandHere = (NvPerlCommand*)pCommand;
	if(ScriptData == NULL)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScript");
		pCommandHere->SysLog("Run script", "", NV_LOG_INFO, 1);
		pCommandHere->SysLog("*** Error - Script data is NULL", "", NV_LOG_ERROR, 1);
		return 0;
	}
	
	DEBUGOUTPUT("NvInterpretorRunScriptMemory");
	NvPerlInterpretor *PerlInterpretor = GetInterpretorFromCallBack();
	if(PerlInterpretor == NULL)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		pCommandHere->SysLog("*** Error - Cannot get interpretor", "", NV_LOG_ERROR, 1);
		return 0;
	}
	PerlInterpretor->LogLevel = pCommandHere->GetLogLevel();
	if(PerlInterpretor->LogLevel)
		pCommandHere->SysLog("Run memory script", "", NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
	PerlInterpretor->m_pCommand = pCommandHere;
	PerlInterpretor->ScriptString = ScriptData;
	int RetVal = 1;
	DEBUGOUTPUT("NvInterpretorRunScriptMemory");
	try
	{
		RetVal = PerlInterpretor->RunScriptMemory();
	}
	catch(...)
	{
		DEBUGERROR("*** NvPerlInterpretor::RunScriptMemory");
		if(PerlInterpretor->LogLevel)
			pCommandHere->SysLog("*** Error - Exception when running script", "", NV_LOG_ERROR, 1, PerlInterpretor->Id.c_str());
		RetVal = 0;
	}
	
	if(PerlInterpretor->LogLevel)
	{
		if(RetVal)
			pCommandHere->SysLog("Script terminated successfuly", "", NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
		else
			pCommandHere->SysLog("*** Script terminated with error", "", NV_LOG_INFO, 1, PerlInterpretor->Id.c_str());
	}

	DEBUGOUTPUT("NvInterpretorRunScriptMemory");
	return RetVal;
}
*/

// Called by perl after initialization
EXTERN_C void
xs_init(pTHX)
{
	char *file = (char *)__FILE__;
	dXSUB_SYS;


	// DynaLoader is a special case
	newXS((char *)"DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
//	newXS("Socket::bootstrap", boot_Socket, file);
	
	// We must install here our callbacks
	newXS((char *)"NV::Command", NV_Command, file);
	newXS((char *)"NV::SetError", NV_SetError, file);
	newXS((char *)"NV::SetErrorEx", NV_SetErrorEx, file);
	newXS((char *)"NV::SetErrorMode", NV_SetErrorMode, file);
	newXS((char *)"NV::GetSessionId", NV_GetSessionId, file);
	newXS((char *)"NV::GetService", NV_GetService, file);
	newXS((char *)"NV::GetClass", NV_GetClass, file);
	newXS((char *)"NV::GetCommand", NV_GetCommand, file);
	newXS((char *)"NV::GetInContainer", NV_GetInContainer, file);
	newXS((char *)"NV::GetOutContainer", NV_GetOutContainer, file);
	newXS((char *)"NV::GetLanguage", NV_GetLanguage, file);
	newXS((char *)"NV::GetError", NV_GetError, file);
	newXS((char *)"NV::GetNumParameters", NV_GetNumParameters, file);
	newXS((char *)"NV::GetSource", NV_GetSource, file);
	newXS((char *)"NV::ParameterExist", NV_ParameterExist, file);
	newXS((char *)"NV::GetParameter", NV_GetParameter, file);
	newXS((char *)"NV::GetParameterName", NV_GetParameterName, file);
	newXS((char *)"NV::SetEncoding", NV_SetEncoding, file);
	newXS((char *)"NV::EvalString", NV_EvalString, file);
}

NvPerlInterpretor* GetInterpretorFromCallBack()
{
	// We get back the pointer to the NvPerlInterpretor class that called the perl script
#ifdef WIN32
	void *ThreadHandle = (void *)GetCurrentThreadId();			// Thread handle for window
#else
	void *ThreadHandle = (void *) pthread_self();				// Thread handle for unix
#endif

	ProtectInterpretors.Lock();
	NvPerlInterpretorList *PerlClassList = NULL;
	if(InterpretorLists.find(ThreadHandle) != InterpretorLists.end())
		PerlClassList = (NvPerlInterpretorList *)(InterpretorLists[ThreadHandle]);
	NvPerlInterpretor *PerlClass = NULL;
	int Size = 0;
	if(PerlClassList != NULL)
		Size = (int)(PerlClassList->m_vInterpretors.size());
	if(Size)
		PerlClass = (NvPerlInterpretor *)(PerlClassList->m_vInterpretors[Size-1]);
	ProtectInterpretors.Unlock();

	if(PerlClass != NULL)
	{
		int CheckResult = 0;
		// Try something to test if the PerClass is OK or not
		try
		{
			CheckResult = PerlClass->Check(1, 2);
		}
		catch(...)
		{
			CheckResult = 0;
		}
		if(!CheckResult)
		{
			DEBUGERROR("*** NvPerlInterpretor::GetInterpretorFromCallBack");
			PerlClass = NULL;
		}
	}
	return PerlClass;
}


// Callback for NIRVA Command function
EXTERN_C
XS(NV_Command)
{
    dXSARGS;
	char *CommandString;

	if(items != 1)
		XSRETURN_NO;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("Start Nirva command", "", NV_LOG_INFO, 3, PerlClass->Id.c_str());
		
	// Get the parameter
	CommandString = (char *)SvPV(ST(0), PL_na);
	if(CommandString == NULL)
	{
		if(!PerlClass->ErrorModeScript)
		{
			PerlClass->GotError = 1;
			if(PerlClass->LogLevel)
				PerlClass->m_pCommand->SysLog("*** Error No command string", "", NV_LOG_ERROR, 1, PerlClass->Id.c_str());
			croak("");
		}
		XSRETURN_NO;
	}
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("Nirva command string", CommandString, NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	int RetVal = 1;
	try
	{
		RetVal = PerlClass->Command(CommandString, &ResBuffer);
	}
	catch(...)
	{
		RetVal = 0;
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (Command)|");
	}
	if(!RetVal)
	{
		if(!PerlClass->ErrorModeScript)
		{
			PerlClass->GotError = 1;
			croak("");
			if(PerlClass->LogLevel)
				PerlClass->m_pCommand->SysLog("Script interrupted", "Error in Nirva command", NV_LOG_INFO, 1, PerlClass->Id.c_str());
		}
		XSRETURN_NO;
	}

	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("Nirva command successful", "", NV_LOG_INFO, 3, PerlClass->Id.c_str());

	XSRETURN_YES;
}



// Callback for NIRVA SetError function
EXTERN_C
XS(NV_SetError)
{
    dXSARGS;
	char *ErrorInfo;

	if(items != 1)
		XSRETURN_NO;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("SetError", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	// Get the parameter
	ErrorInfo = (char *)SvPV(ST(0), PL_na);
	if(ErrorInfo == NULL)
		XSRETURN_NO;

	try
	{
		PerlClass->SetError(ErrorInfo);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (SetError)|");
		XSRETURN_NO;
	}


	XSRETURN_YES;
}



// Callback for NIRVA SetErrorEx function
EXTERN_C
XS(NV_SetErrorEx)
{
    dXSARGS;
	char *ErrorService;
	char *ErrorClass;
	char *ErrorInfo;
	int ErrorCode = 0;

	if(items < 3)
		XSRETURN_NO;



	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("SetErrorEx", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	// Get the error service
	ErrorService = (char *)SvPV(ST(0), PL_na);
	if(ErrorService == NULL)
		XSRETURN_NO;

	// Get the error class
	ErrorClass = (char *)SvPV(ST(1), PL_na);
	if(ErrorClass == NULL)
		XSRETURN_NO;

	// Get the error code
	ErrorCode = (int)(SvIV(ST(2)));

	if(items > 3)
	{
		// Get the error info
		ErrorInfo = (char *)SvPV(ST(3), PL_na);
		if(ErrorInfo == NULL)
			XSRETURN_NO;
	}

	try
	{
		if(items > 3)
			PerlClass->SetErrorEx(ErrorService, ErrorClass, ErrorCode, ErrorInfo);
		else
			PerlClass->SetErrorEx(ErrorService, ErrorClass, ErrorCode, "");
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (SetErrorEx)|");
		XSRETURN_NO;
	}


	XSRETURN_YES;
}






// Sets the error mode "SCRIPT" for control to scrip and "NIRVA" for control by NIRVA
EXTERN_C
XS(NV_SetErrorMode)
{
    dXSARGS;
	char *ErrorMode;

	int ErrorModeScriptHere = 0;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;


	// Get the parameter
	string ErrorModeHere = "";
	if(items > 0)
	{
		ErrorMode = (char *)SvPV(ST(0), PL_na);
		if(ErrorMode != NULL)
		{
			ErrorModeHere = ErrorMode;
			MakeUpper(ErrorModeHere);
			CleanString(ErrorModeHere);
			if(ErrorModeHere == "SCRIPT")
				ErrorModeScriptHere = 1;
		}
	}

	PerlClass->ErrorModeScript = ErrorModeScriptHere;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("SetErrorMode", ErrorModeHere.c_str(), NV_LOG_INFO, 4, PerlClass->Id.c_str());

	XSRETURN_YES;
}




// Callback for NIRVA GetSessionId function
EXTERN_C
XS(NV_GetSessionId)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetSessionId", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	try
	{
		PerlClass->GetSessionId(&ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetSessionId)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);



	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}




// Callback for NIRVA GetService function
EXTERN_C
XS(NV_GetService)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetService", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	try
	{
		PerlClass->GetService(&ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetService)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}




// Callback for NIRVA GetClass function
EXTERN_C
XS(NV_GetClass)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetClass", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	try
	{
		PerlClass->GetClass(&ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetClass)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}




// Callback for NIRVA GetCommand function
EXTERN_C
XS(NV_GetCommand)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetCommand", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	try
	{
		PerlClass->GetCommand(&ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetCommand)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}




// Callback for NIRVA GetInContainer function
EXTERN_C
XS(NV_GetInContainer)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetInContainer", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	try
	{
		PerlClass->GetInContainer(&ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetInContainer)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}




// Callback for NIRVA GetOutContainer function
EXTERN_C
XS(NV_GetOutContainer)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetOutContainer", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	try
	{
		PerlClass->GetOutContainer(&ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetOutContainer)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}





// Callback for NIRVA GetLanguage function
EXTERN_C
XS(NV_GetLanguage)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetLanguage", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	char *ResBuffer;

	try
	{
		PerlClass->GetLanguage(&ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetLanguage)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}


// Callback for NIRVA GetError function
EXTERN_C
XS(NV_GetError)
{
    dXSARGS;
	char *InfoType;


	if(items != 1)
		XSRETURN_NO;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetError", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	// Get the parameter
	InfoType = (char *)SvPV(ST(0), PL_na);
	if(InfoType == NULL)
		XSRETURN_NO;
	

	char *ResBuffer;

	try
	{
		PerlClass->GetError(InfoType, &ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetError)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}


// Callback for NIRVA GetNumParameters function
EXTERN_C
XS(NV_GetNumParameters)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetNumParameters", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	int RetVal = 0;

	try
	{
		RetVal = PerlClass->GetNumParameters();
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetNumParameters)|");
	}

	XSRETURN_IV(RetVal);

//	if(!RetVal)
//		XSRETURN_NO;
	
//	XSRETURN_YES;
}


// Callback for NIRVA GetSource function
EXTERN_C
XS(NV_GetSource)
{
    dXSARGS;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetSource", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());
	
	int SourceType = -1;

	try
	{
		SourceType = PerlClass->GetSource();
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetSource)|");
	}


	string sResult = "UNKNOWN";
	switch(SourceType)
	{
		case NV_SOURCE_CLIENT:
			sResult = "CLIENT";
			break;
		case NV_SOURCE_BROWSER:
			sResult = "BROWSER";
			break;
		case NV_SOURCE_PROCEDURE:
			sResult = "PROCEDURE";
			break;
		case NV_SOURCE_SERVICE:
			sResult = "SERVICE";
			break;
		case NV_SOURCE_SERVER:
			sResult = "SERVER";
			break;
	}

	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, sResult.c_str());
	

	XSRETURN_YES;
}




// Callback for NIRVA ParameterExist function
EXTERN_C
XS(NV_ParameterExist)
{
    dXSARGS;
	char *ParameterName;

	if(items != 1)
		XSRETURN_NO;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("ParameterExist", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	// Get the parameter
	ParameterName = (char *)SvPV(ST(0), PL_na);
	if(ParameterName == NULL)
		XSRETURN_NO;
	
	int RetVal = 0;

	try
	{
		RetVal = PerlClass->ParameterExist(ParameterName);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (ParameterExist)|");
	}



	if(!RetVal)
		XSRETURN_NO;
	
	XSRETURN_YES;
}



// Callback for NIRVA GetParameter function
EXTERN_C
XS(NV_GetParameter)
{
    dXSARGS;
	char *ParameterName;

	if(items != 1)
		XSRETURN_NO;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetParameter", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	// Get the parameter
	ParameterName = (char *)SvPV(ST(0), PL_na);
	if(ParameterName == NULL)
		XSRETURN_NO;
	

	char *ResBuffer;

	try
	{
		PerlClass->GetParameter(ParameterName, &ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetParameter)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}



// Callback for NIRVA GetParameterName function
EXTERN_C
XS(NV_GetParameterName)
{
    dXSARGS;
	int ParamIndex = -1;

	if(items != 1)
		XSRETURN_NO;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("GetParameterName", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	// Get the parameter
	ParamIndex = (int)(SvIV(ST(0)));
	

	char *ResBuffer;

	try
	{
		PerlClass->GetParameterName(ParamIndex, &ResBuffer);
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (GetParameterName)|");
	}


	// Here is the way to set a string variable to the perl script
	SV* SVResult = get_sv("NV::RESULT", TRUE);
	sv_setpv(SVResult, ResBuffer);
	

	XSRETURN_YES;
}





// Callback for NIRVA SetEncoding function
EXTERN_C
XS(NV_SetEncoding)
{
    dXSARGS;
	char *Encoding;

	if(items != 1)
		XSRETURN_NO;

	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("SetEncoding", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

	// Get the parameter
	Encoding = (char *)SvPV(ST(0), PL_na);
	if(Encoding == NULL)
		XSRETURN_NO;
	
	int RetVal = 0;

	try
	{
		PerlClass->SetEncoding(Encoding);
		RetVal = 1;
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (SetEncoding)|");
	}



	if(!RetVal)
		XSRETURN_NO;
	
	XSRETURN_YES;
}

// Callback for NIRVA GetCommand function
EXTERN_C
XS(NV_EvalString)
{
    dXSARGS;


	NvPerlInterpretor *PerlClass = GetInterpretorFromCallBack();
	if(PerlClass == NULL)
		XSRETURN_NO;

	bool RetVal = true;
	PerlClass->EvalError = "";
	
	if(PerlClass->LogLevel)
		PerlClass->m_pCommand->SysLog("EvalString", "", NV_LOG_INFO, 4, PerlClass->Id.c_str());

#ifdef USE_TRY_CATCH
	try
	{
#endif
		ENTER;
		SAVETMPS;
		SV *code = newSVpv(PerlClass->ScriptString.c_str(), 0);
		eval_sv(code, G_EVAL|G_SCALAR|G_KEEPERR);
		SvREFCNT_dec(code);
//		SV *value = (eval_pv(PerlClass->ScriptString.c_str(), FALSE));
		if(SvTRUE(ERRSV))
		{
			PerlClass->EvalError = SvPV(ERRSV,PL_na);
			if(PerlClass->EvalError.length() && (PerlClass->EvalError[PerlClass->EvalError.length()-1] == '\n'))
				PerlClass->EvalError = PerlClass->EvalError.substr(0, PerlClass->EvalError.length()-1);
			if(PerlClass->bDebug)
				fprintf(stderr, "%s\n", PerlClass->EvalError.c_str());
			RetVal = false;
		}
		FREETMPS;
		LEAVE;
#ifdef USE_TRY_CATCH
	}
	catch(...)
	{
		PerlClass->m_pCommand->Command("NV_CMD=|SYSTEM:ON_EXCEPTION| MESSAGE=|Exception occured in perl interface (Eval)|");
		RetVal = false;
	}
#endif

	if(!RetVal)
		XSRETURN_NO;
	
	XSRETURN_YES;
}

