// ***************************************************************************************
// File: dotnetint.h
// Description: interface to dotnet
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _DOTNETINT_H
#define _DOTNETINT_H

#include "task.h"
#include "util.h"
#include "../nvdotnet/edotnet.h"

// Classes declared in this file
class NvDotnet;
class NvDotnetMethod;


// NvDotnet class. Interface to dotnet. There is only one instance of this class in the main nvs class

class NvDotnet
{
public:

	NvMutex ProtectDotnet;						// Mutex for dotnet interface protection
	HMODULE hLibDotnet;							// Handle to the nvdotnet dynamic library
	DOTNETINIT DotnetLibInit;						// Dotnet library InitDotnet function pointer
	DOTNETFREE DotnetLibFree;						// Dotnet library FreeDotnet function pointer
	DOTNETRUNCLASS DotnetLibRunClass;				// Dotnet library RunClass function pointer
	DOTNETUNLOADDOMAIN DotnetLibUnloadDomain;		// Dotnet library UnloadDomain function pointer
	int NumInstances;
	bool m_bAvailable;


	NvDotnet();
	virtual ~NvDotnet();

	bool LoadDotnetLib(const char *NirvaDir, bool UseLocking = true);
	void FreeDotnetLib(bool UseLocking = true);
	void UnloadDomain(const char* Type, const char* Name);
	bool IsDotnetLibLoaded(bool UseLocking = true);
	int RunMethod(const char* Method, void *pCommand);
	int ThreadRunMethod(const char* Method, void *pCommand, bool SameThread = false);
};


// Dotnet method instance
class NvDotnetMethod : public NvTask
{
// Attributes
public:
	const char* MethodPath;
	void *pCommandHere;
	bool b_Initialized;

	NvDotnetMethod();

	// Overridable callbacks
protected:
	virtual int Main();
	virtual bool OnInit();
	virtual void OnCleanup();

public:
	bool IsDotnetAvailable();
	int RunMethod(const char* Method, void *pCommand, bool SameThread = false);
};



#endif //_DOTNETINT_H
