// ***************************************************************************************
// File: javaint.h
// Description: java interface
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _JAVAINT_H
#define _JAVAINT_H

#include "task.h"
#include "util.h"
#include "../nvjava/ejava.h"

// Classes declared in this file
class NvJava;
class NvJavaMethod;


// NvJava class. Interface to java. There is only one instance of this class in the main nvs class

class NvJava
{
public:

	NvMutex ProtectJava;						// Mutex for java interface protection
	HMODULE hLibJava;							// Handle to the nvjava dynamic library
	JAVAINIT JavaLibInit;						// Java library InitJava function pointer
	JAVAFREE JavaLibFree;						// Java library FreeJava function pointer
	JAVARUNMETHOD JavaLibRunMethod;				// Java library RunMethod function pointer
	JAVADETACHTHREAD JavaLibDetachThread;		// Java library DetachThread function pointer
	JAVAUNLOADENV JavaLibUnloadEnv;				// Java library DetachThread function pointer
	int NumInstances;
	bool m_bAvailable;


	NvJava();
	virtual ~NvJava();

	bool LoadJavaLib(const char *NirvaDir, int MinMemSize = 2, int MaxMemSize = 64, bool UseLocking = true, const char* JavaHomeDir = NULL, const char* JvmPath = NULL, const char* JvmParam = NULL, const char* ExtraParam = NULL);
	void FreeJavaLib(bool UseLocking = true);
	void UnloadEnv(const char* Type, const char* Name);
	void CleanJavaLib(const char *NirvaDir, int MinMemSize, int MaxMemSize, const char* JavaHomeDir = NULL, const char* JvmPath = NULL, const char* JvmParam = NULL, const char* ExtraParam = NULL);
	bool IsJavaLibLoaded(bool UseLocking = true);
	void DetachJavaThread();
	int RunMethod(const char* Method, void *pCommand);
	int ThreadRunMethod(const char* Method, void *pCommand, bool SameThread = false);
};


// Java method instance
class NvJavaMethod : public NvTask
{
// Attributes
public:
	const char* MethodPath;
	void *pCommandHere;
	bool b_Initialized;

	NvJavaMethod();

	// Overridable callbacks
protected:
	virtual int Main();
	virtual bool OnInit();
	virtual void OnCleanup();

public:
	bool IsJavaAvailable();
	int RunMethod(const char* Method, void *pCommand, bool SameThread = false);
};



#endif //_JAVAINT_H
