// ***************************************************************************************
// File: mqint.h
// Description: interface to websphere mq
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _MQINT_H
#define _MQINT_H

#include "task.h"
#include "util.h"
#include "../nvmq/emq.h"

// Classes declared in this file
class NvMqLib;
class MyNvMqInMsg;
class MyNvMqOutMsg;
class NvMqThread;


// NvMqLib class. Interface to mq. There is only one instance of this class in the main nvs class

class NvMqLib
{
public:

	NvMutex ProtectMq;							// Mutex for mq interface protection
	HMODULE hLibMq;								// Handle to the nvmq dynamic library
	INITMQLIB MqLibInit;						// Mq library InitMqLib function pointer
	EXITMQLIB MqLibFree;						// Mq library ExitMqLib function pointer
	MQ_OPEN MqOpen;								// Mq library Open function pointer
	MQ_CLOSE MqClose;							// Mq library Close function pointer
	MQ_GET MqGet;								// Mq library Get function pointer
	MQ_PUT MqPut;								// Mq library Put function pointer
	bool m_bAvailable;


	NvMqLib();
	virtual ~NvMqLib();

	bool LoadMqLib(const char *NirvaDir, bool UseLocking = true);
	void FreeMqLib(bool UseLocking = true);
	bool IsMqLibLoaded(bool UseLocking = true);
};



class MyNvMqInMsg : public NvMqInMsg
{
public:
	int DataSize;
	char *Data;
	unsigned char MessageId[24];
	unsigned char CorrelationId[24];
	bool Reply;
	int Persistence;
	bool CopyMessageId;
	bool CopyCorrelationId;
	std::string ReplyQueueName;
	int Type;
	std::string Parameters;
	int Expiry;

	MyNvMqInMsg();
	virtual ~MyNvMqInMsg();

	void Clear();
	char* GetData(){return Data;};
	char* SetData(const char* Buffer, int BufferSize);
	int GetDataSize(){return DataSize;};
	unsigned char* GetMessageId(){return MessageId;};
	void SetMessageId(const unsigned char* Buffer, int BufferSize);
	unsigned char* GetCorrelationId(){return CorrelationId;};
	void SetCorrelationId(const unsigned char* Buffer, int BufferSize);
	void SetReply(bool Value){Reply=Value;};
	bool GetReply(){return Reply;};
	void SetPersistence(int Value){Persistence=Value;};
	int GetPersistence(){return Persistence;};
	void SetExpiry(int Value){Expiry=Value;};
	int GetExpiry(){return Expiry;};
	void SetCopyMessageId(bool Value){CopyMessageId=Value;};
	bool GetCopyMessageId(){return CopyMessageId;};
	void SetCopyCorrelationId(bool Value){CopyCorrelationId=Value;};
	bool GetCopyCorrelationId(){return CopyCorrelationId;};
	void SetReplyQueueName(const char* Value){ReplyQueueName=Value;};
	std::string& GetReplyQueueName(){return ReplyQueueName;};
	void SetType(int Value){Type=Value;};
	int GetType(){return Type;};
	void SetParameters(const char* Value){Parameters=Value;};
	std::string& GetParameters(){return Parameters;};
};



class MyNvMqOutMsg : public NvMqOutMsg
{
public:
	int DataSize;
	char *Data;
	unsigned char InputMessageId[24];
	unsigned char InputCorrelationId[24];
	int Persistence;
	bool CopyMessageId;
	bool CopyCorrelationId;
	bool WithMessageHeader;
	std::string QueueName;
	int Type;
	int ErrorCode;
	std::string ErrorService;
	std::string ErrorClass;
	std::string ErrorInfo;
	std::string ErrorDesc;
	int Expiry;

	MyNvMqOutMsg();
	virtual ~MyNvMqOutMsg();
	void Init(MyNvMqInMsg *InMsg);

	void Clear();
	char* GetData(){return Data;};
	char* SetData(const char* Buffer, int BufferSize);
	int GetDataSize(){return DataSize;};
	unsigned char* GetInputMessageId(){return InputMessageId;};
	void SetInputMessageId(const unsigned char* Buffer, int BufferSize);
	unsigned char* GetInputCorrelationId(){return InputCorrelationId;};
	void SetInputCorrelationId(const unsigned char* Buffer, int BufferSize);
	void SetPersistence(int Value){Persistence=Value;};
	int GetPersistence(){return Persistence;};
	void SetExpiry(int Value){Expiry=Value;};
	int GetExpiry(){return Expiry;};
	void SetCopyMessageId(bool Value){CopyMessageId=Value;};
	bool GetCopyMessageId(){return CopyMessageId;};
	void SetCopyCorrelationId(bool Value){CopyCorrelationId=Value;};
	bool GetCopyCorrelationId(){return CopyCorrelationId;};
	void SetWithMessageHeader(bool Value){WithMessageHeader=Value;};
	bool GetWithMessageHeader(){return WithMessageHeader;};
	void SetQueueName(const char* Value){QueueName=Value;};
	std::string& GetQueueName(){return QueueName;};
	void SetType(int Value){Type=Value;};
	int GetType(){return Type;};
	void SetErrorCode(int Value){ErrorCode=Value;};
	int GetErrorCode(){return ErrorCode;};
	void SetErrorService(const char* Value){ErrorService=Value;};
	std::string& GetErrorService(){return ErrorService;};
	void SetErrorClass(const char* Value){ErrorClass=Value;};
	std::string& GetErrorClass(){return ErrorClass;};
	void SetErrorInfo(const char* Value){ErrorInfo=Value;};
	std::string& GetErrorInfo(){return ErrorInfo;};
	void SetErrorDesc(const char* Value){ErrorDesc=Value;};
	std::string& GetErrorDesc(){return ErrorDesc;};

};


class NvMqThread : public NvTask
{
public:

	NvMqLib *MqLib;							// library management for mq
	std::string QueueManagerName;			// Queue manager name
	std::string QueueName;					// Queue name
	std::string Filter;						// Queue message filter
	std::string Channel;					// Queue manager channel name
	std::string Server;						// Queue manager server address
	MyNvMqInMsg InputMessage;
	MyNvMqOutMsg OutputMessage;
	NvCommand Command;						// Client current command
	NvSession * m_sSession;					// Current session
	bool RemoveSessionOnCleanup;			// If this flag is set, the session is removed on cleanup
	bool b_Initialized;
	bool RequestToLeave;					// If this flag is set, the session is removed on cleanup
	NVMQIDENT MqIdent;						// MQ identifier for the instance in the nvmq library


	NvMqThread();


// Overridable callbacks
protected:
	virtual int Main();
	virtual void OnException(std::string sInfo);
	virtual bool OnInit();
	virtual void OnCleanup();

public:
	// Connects the queue
	bool Connect();
	
	// Disonnects the queue
	void Disconnect();
	
	// Stops the task
	void Stop();
	
	// Sets the actual session
	void SetSession(NvSession *Session);
	
	// Gets the actual session
	NvSession* GetSession();

	// Process a received message
	void ProcessMessage();

	// Set the error if there is one
	void SetErrorCommand(int XmlCoding);


	bool ParseXmlCommand(NvObjBinary *XmlDataSource = NULL);
	bool GetSoapBody();
	bool GetSoapCommandHeader(NvObjBinary *SoapCommandHeader);
};






class NvMq : public NvObject
{
public:
	std::list<NvMqThread *> m_vTheads;		// queue threads
	int m_iNumThreads;						// Number of threads that listen on the queue

	NvMqLib *MqLib;							// library management for mq
	std::string Name;						// Queue manager name
	std::string Ident;						// Queue manager name
	std::string Description;				// Queue manager name
	std::string QueueManagerName;			// Queue manager name
	std::string QueueName;					// Queue name
	std::string Filter;						// Queue message filter
	std::string Channel;					// Queue manager channel name
	std::string Server;						// Queue manager server address
	bool m_bRunning;						// Running or stopped flag
	bool m_bAutoStart;						// Auto start flag

	NvMq();
	virtual ~NvMq();

public:
	// Start listening
	bool Start();
	
	// Stop listening
	bool Stop(bool Force = false, int WaitTime = 10);

	// Get running state
	bool IsRunning();
};


class NvMqList : public NvObject
{
protected:
	NvMq* GetQueue(const char *Name);

public:
	std::map<std::string, NvMq*> m_mQueues;
	NvMqLib *MqLib;							// library management for mq

	NvMqList();
	virtual ~NvMqList();

	static void CleanQueueName(std::string& QueueName);
	static std::string GetQueueIdent(std::string QueueName);

	// Removes all queues from the list
	void Clear();
	
	// Load the queue list from registry
	void Load();

	// Saves the queue list to the system registry
	bool Save();
	
	// Creates a new queue
	bool CreateQueue(const char *Name, const char *Description = NULL, const char *QueueManagerName = NULL, const char *QueueName = NULL, const char *Filter = NULL, int NumThreads = -1, int AutoStart = -1, const char *Channel = NULL, const char *Server = NULL);
	
	// Set queue parameters
	bool SetQueueParam(const char *Name, const char *Description = NULL, const char *QueueManagerName = NULL, const char *QueueName = NULL, const char *Filter = NULL, int NumThreads = -1, int AutoStart = -1, const char *Channel = NULL, const char *Server = NULL);

	// Removes a queue
	bool RemoveQueue(const char *Name);

	// Start a queue
	bool StartQueue(const char *Name);

	// Stop a queue
	bool StopQueue(const char *Name, bool Force = false, int WaitTime = 10);

	// Test the queue running state
	bool IsQueueRunning(const char *Name);

	// Get queue list
	bool QueueList(const char *Name, NvObjTable *QueueInfo);

	// Start all queues with autostart flag
	bool AutoStartQueues();

	// Stop all started queues
	bool StopQueues();

	// Test if a queue exist
	bool QueueExist(const char *Name);
};





#endif //_MQINT_H
