// ***************************************************************************************
// File: package.h
// Description: Installation packages.
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _PACKAGE_H
#define _PACKAGE_H

// Classes declared in this file
class NvPackage;

#define PACKAGE_FILE_IDENTIFIER		"NIRVA PACKAGE FILE"
#define PACKAGE_FILE_VERSION		"1"




/////////////////////////////////////////////////////////////////////////////
// NvPackage

class NvPackage
{
public:
	std::string Info;
	
	NvPackage();
	virtual ~NvPackage();

	bool AddFile(std::string FileName, std::string FileSource, FILE* PkgFile, bool bSourceIsDir, bool bWithSubDir, bool bRemoveDir, bool bTextDir, std::string RelativeDir, std::vector<std::string> *ExcludeFiles = NULL, std::vector<std::string> *ExcludeDirs = NULL, bool bPerlEncode = false);
	bool ProcessFileLines(std::vector<std::string> *Lines, FILE* PkgFile, bool bPerlEncode = false);
	bool ProcessPerlFileLines(std::vector<std::string> *Lines, FILE* PkgFile);
	bool ProcessDirLines(std::vector<std::string> *Lines, std::vector<std::string> *ExcludeFiles, FILE* PkgFile, std::string RelativeDir, bool bPerlEncode = false);
	bool ProcessDirSubLines(std::vector<std::string> *Lines, std::vector<std::string> *ExcludeFiles, std::vector<std::string> *ExcludeDirs, FILE* PkgFile, std::string RelativeDir, bool bPerlEncode = false);
	bool ProcessRemoveDirLines(std::vector<std::string> *Lines, FILE* PkgFile);
	bool ProcessRemoveDirSubLines(std::vector<std::string> *Lines, FILE* PkgFile);
	bool ProcessExecLines(std::vector<std::string> *Lines, FILE* PkgFile);
	bool ProcessTextFileLines(std::vector<std::string> *Lines, FILE* PkgFile);
	bool ProcessTextDirLines(std::vector<std::string> *Lines, FILE* PkgFile);

	bool Check(const char *DescriptionFile, const char *BaseDir);
	bool Create(const char *DescriptionFile, const char *BaseDir, const char *PackageFile = NULL);
	bool Install(const char *PackageFile, const char *BaseDir, bool WithInfo = false);
	bool GetHeaderLines(const char *PackageFile, std::vector<std::string> *Lines);
	bool SetTextFile(const char *FileName);
	bool AppendPerlFile(FILE* ObjSourceFile, FILE* PkgFile, std::string& FileName);
};


#endif //_PACKAGE_H
