// ***************************************************************************************
// File: security.h
// Description: security classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _SECURITY_H
#define _SECURITY_H


// Classes declared in this file
class NvSecurity;

#include "task.h"
#include "obj.h"
#include <string>




class NvSecurity : public NvObject
{
public:

	NvObjTable	m_Users;				// user list
	NvObjTable	m_Roles;				// role list
	NvApplication *m_Application;		// application (NULL if system)
	int iDefaultPasswordExpire;			// Default expiration time for passwords in days (-1 = infinite)
	bool bDefaultPasswordChangeNext;	// Default flag for changing password at next logon
	int iUserTimeOut;					// Time out for unused users (-1 = infinite)
	int iLockingAttempts;				// Number of attemps before locking (-1 = no locking)
	std::string sProcCheckPassword;		// Procedure for checking password
	
	NvSecurity();
	virtual ~NvSecurity();

	// Internal functions
	void Clear();
	int GetUserIndex(const char* UserName);
	int GetRoleIndex(const char* RoleName);
	bool GetRolePermissions(const char* RoleName, NvObjIndStringList* ProcessedRoles, NvObjIndStringList* Permissions);
	
	
	bool Load();
	bool Save(bool lockit = true);
	bool Export(const char* FileName);
	bool Import(const char* FileName);

	int Login(const char *UserName, const char *UserPassword, const char *NewPassword, const char *NewPasswordConfirm, bool bWithNewPassword, bool bWithNewPasswordConfirm, NvObjIndStringList *UserPermissions, std::string& UserFullName, bool ControlPassword = true, bool NewPassordOk = true);

	bool SetPasswordExpire(int PasswordExpire, bool ApplyToAll = false);
	int GetPasswordExpire();
	bool SetPasswordChangeNext(bool PasswordChangeNext, bool ApplyToAll = false);
	bool GetPasswordChangeNext();
	bool SetUserTimeOut(int UserTimeOut);
	int GetUserTimeOut();
	bool SetLockingAttempts(int LockingAttempts);
	int GetLockingAttempts();
	bool SetProcCheckPassword(std::string ProcCheckPassword);
	std::string GetProcCheckPassword();

	bool AddUser(const char* UserName, const char* Description = NULL, const char* FullName = NULL, const char* Password = NULL, const char* PasswordExpire = NULL, const char* PasswordChangeFirst = NULL);
	bool RemoveUser(const char* UserName);
	bool EnableUser(const char* UserName, bool Enable = true);
	bool UserList(NvObjTable *Users, const char* UserName = NULL);
	bool UserExist(const char* UserName);
	bool SetUserDescription(const char* UserName, const char* Description);
	std::string GetUserDescription(const char* UserName);
	bool SetUserFullName(const char* UserName, const char* FullName);
	std::string GetUserFullName(const char* UserName);
	bool SetUserPassword(const char* UserName, const char* Password);
	std::string GetUserPassword(const char* UserName);
	bool SetUserPasswordExpire(const char* UserName, int PasswordExpire);
	int GetUserPasswordExpire(const char* UserName);
	bool SetUserPasswordChangeNext(const char* UserName, bool PasswordChangeNext);
	bool GetUserPasswordChangeNext(const char* UserName);
	bool SetUserRoles(const char* UserName, const char* Roles, bool Append = true);
	bool RemoveUserRoles(const char* UserName, const char* Roles, bool All = false);
	bool GetUserPermissions(const char* UserName, NvObjIndStringList* Permissions);
	bool IsUserEnabled(const char* UserName);
	bool IsPasswordExpired(const char* UserName);
	bool LoadUserContext(const char* UserName, NvContainer *Container);
	bool SaveUserContext(const char* UserName, NvContainer *Container);
	bool RemoveUserContext(const char* UserName);

	bool AddRole(const char* RoleName, const char* Description = NULL);
	bool RemoveRole(const char* RoleName);
	bool RoleList(NvObjTable *Roles, const char* RoleName = NULL);
	bool RoleExist(const char* RoleName);
	bool SetRoleDescription(const char* RoleName, const char* Description);
	std::string GetRoleDescription(const char* RoleName);
	bool SetRoleInherits(const char* RoleName, const char* Inherits, bool Append = true);
	bool RemoveRoleInherits(const char* RoleName, const char* Inherits, bool All = false);
	bool SetRolePermissions(const char* RoleName, const char* Permissions, bool Append = true);
	bool RemoveRolePermissions(const char* RoleName, const char* Permissions, bool All = false);


};


#endif //_SECURITY_H
